# coding: utf-8
from __future__ import print_function, division, unicode_literals, absolute_import

print(
    "You mean the perl language i.e. the only language that looks the same before and after RSA encryption?\n"
    "No way, use this instead\n")

import this
